]> sigrok.org Git - libsigrok.git/blobdiff - std.c
Doxygen: Consistently use @ notation everywhere.
[libsigrok.git] / std.c
diff --git a/std.c b/std.c
index 8add0d3dc622c2045cfa5c870efb4ab39183dde7..b6b6f6c7417667ab28633546e227bbdeec50ff5c 100644 (file)
--- a/std.c
+++ b/std.c
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
+/** @file
+  * Standard API helper functions.
+  * @internal
+  */
+
 #include <glib.h>
 #include "libsigrok.h"
 #include "libsigrok-internal.h"
 /**
  * Standard sr_driver_init() API helper.
  *
- * This function can be used to simplify most driver's hw_init() API callback.
+ * This function can be used to simplify most driver's init() API callback.
  *
  * It creates a new 'struct drv_context' (drvc), assigns sr_ctx to it, and
  * then 'drvc' is assigned to the 'struct sr_dev_driver' (di) that is passed.
  *
  * @param sr_ctx The libsigrok context to assign.
  * @param di The driver instance to use.
- * @param prefix A driver-specific prefix string used for log messages.
+ * @param[in] prefix A driver-specific prefix string used for log messages.
  *
  * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
  *         SR_ERR_MALLOC upon memory allocation errors.
  */
-SR_PRIV int std_hw_init(struct sr_context *sr_ctx, struct sr_dev_driver *di,
-                       const char *prefix)
+SR_PRIV int std_init(struct sr_context *sr_ctx, struct sr_dev_driver *di,
+                    const char *prefix)
 {
        struct drv_context *drvc;
 
@@ -63,7 +68,7 @@ SR_PRIV int std_hw_init(struct sr_context *sr_ctx, struct sr_dev_driver *di,
  * Standard API helper for sending an SR_DF_HEADER packet.
  *
  * This function can be used to simplify most driver's
- * hw_dev_acquisition_start() API callback.
+ * dev_acquisition_start() API callback.
  *
  * @param sdi The device instance to use.
  * @param prefix A driver-specific prefix string used for log messages.
@@ -101,27 +106,81 @@ SR_PRIV int std_session_send_df_header(const struct sr_dev_inst *sdi,
        return SR_OK;
 }
 
+#ifdef HAVE_LIBSERIALPORT
+
+/*
+ * Standard serial driver dev_open() helper.
+ *
+ * This function can be used to implement the dev_open() driver API
+ * callback in drivers that use a serial port. The port is opened
+ * with the SERIAL_RDWR and SERIAL_NONBLOCK flags.
+ *
+ * If the open succeeded, the status field of the given sdi is set
+ * to SR_ST_ACTIVE.
+ *
+ * @retval SR_OK Success.
+ * @retval SR_ERR Serial port open failed.
+ */
+SR_PRIV int std_serial_dev_open(struct sr_dev_inst *sdi)
+{
+       struct sr_serial_dev_inst *serial;
+
+       serial = sdi->conn;
+       if (serial_open(serial, SERIAL_RDWR | SERIAL_NONBLOCK) != SR_OK)
+               return SR_ERR;
+
+       sdi->status = SR_ST_ACTIVE;
+
+       return SR_OK;
+}
+
+/*
+ * Standard serial driver dev_close() helper.
+ *
+ * This function can be used to implement the dev_close() driver API
+ * callback in drivers that use a serial port.
+ *
+ * After closing the port, the status field of the given sdi is set
+ * to SR_ST_INACTIVE.
+ *
+ * @retval SR_OK Success.
+ */
+SR_PRIV int std_serial_dev_close(struct sr_dev_inst *sdi)
+{
+       struct sr_serial_dev_inst *serial;
+
+       serial = sdi->conn;
+       if (serial && sdi->status == SR_ST_ACTIVE) {
+               serial_close(serial);
+               sdi->status = SR_ST_INACTIVE;
+       }
+
+       return SR_OK;
+}
+
 /*
  * Standard sr_session_stop() API helper.
  *
  * This function can be used to simplify most (serial port based) driver's
- * hw_dev_acquisition_stop() API callback.
+ * dev_acquisition_stop() API callback.
  *
  * @param sdi The device instance for which acquisition should stop.
  *            Must not be NULL.
  * @param cb_data Opaque 'cb_data' pointer. Must not be NULL.
* @param hw_dev_close_fn Function pointer to the driver's hw_dev_close().
  *                       Must not be NULL.
  * @param serial The serial device instance (struct serial_dev_inst *).
  *               Must not be NULL.
  * @param prefix A driver-specific prefix string used for log messages.
  *               Must not be NULL. An empty string is allowed.
  *
* @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
*         SR_ERR upon other errors.
  */
-SR_PRIV int std_hw_dev_acquisition_stop_serial(struct sr_dev_inst *sdi,
-                       void *cb_data, dev_close_t hw_dev_close_fn,
+SR_PRIV int std_serial_dev_acquisition_stop(struct sr_dev_inst *sdi,
+                       void *cb_data, dev_close_t dev_close_fn,
                        struct sr_serial_dev_inst *serial, const char *prefix)
 {
        int ret;
@@ -134,17 +193,17 @@ SR_PRIV int std_hw_dev_acquisition_stop_serial(struct sr_dev_inst *sdi,
 
        if (sdi->status != SR_ST_ACTIVE) {
                sr_err("%sDevice inactive, can't stop acquisition.", prefix);
-               return SR_ERR;
+               return SR_ERR_DEV_CLOSED;
        }
 
        sr_dbg("%sStopping acquisition.", prefix);
 
-       if ((ret = sr_source_remove(serial->fd)) < 0) {
+       if ((ret = serial_source_remove(serial)) < 0) {
                sr_err("%sFailed to remove source: %d.", prefix, ret);
                return ret;
        }
 
-       if ((ret = hw_dev_close_fn(sdi)) < 0) {
+       if ((ret = dev_close_fn(sdi)) < 0) {
                sr_err("%sFailed to close device: %d.", prefix, ret);
                return ret;
        }
@@ -161,6 +220,8 @@ SR_PRIV int std_hw_dev_acquisition_stop_serial(struct sr_dev_inst *sdi,
        return SR_OK;
 }
 
+#endif
+
 /*
  * Standard driver dev_clear() helper.
  *
@@ -172,42 +233,50 @@ SR_PRIV int std_hw_dev_acquisition_stop_serial(struct sr_dev_inst *sdi,
  * there cannot be freed.
  *
  * @param driver The driver which will have its instances released.
+ * @param clear_private If not NULL, this points to a function called
+ * with sdi->priv as argument. The function can then clear any device
+ * instance-specific resources kept there. It must also clear the struct
+ * pointed to by sdi->priv.
  *
  * @return SR_OK on success.
  */
 SR_PRIV int std_dev_clear(const struct sr_dev_driver *driver,
                std_dev_clear_t clear_private)
 {
-       struct sr_dev_inst *sdi;
        struct drv_context *drvc;
-       struct dev_context *devc;
+       struct sr_dev_inst *sdi;
        GSList *l;
        int ret;
 
-       drvc = driver->priv;
+       if (!(drvc = driver->priv))
+               /* Driver was never initialized, nothing to do. */
+               return SR_OK;
+
        ret = SR_OK;
        for (l = drvc->instances; l; l = l->next) {
-               /* Log errors, but continue cleaning up the rest. */
                if (!(sdi = l->data)) {
                        ret = SR_ERR_BUG;
                        continue;
                }
-               if (!(devc = sdi->priv)) {
-                       ret = SR_ERR_BUG;
-                       continue;
-               }
                if (driver->dev_close)
                        driver->dev_close(sdi);
 
                if (sdi->conn) {
+#if HAVE_LIBSERIALPORT
+                       if (sdi->inst_type == SR_INST_SERIAL)
+                               sr_serial_dev_inst_free(sdi->conn);
+#endif
+#if HAVE_LIBUSB_1_0
                        if (sdi->inst_type == SR_INST_USB)
                                sr_usb_dev_inst_free(sdi->conn);
-                       else if (sdi->inst_type == SR_INST_SERIAL)
-                               sr_serial_dev_inst_free(sdi->conn);
+#endif
+                       if (sdi->inst_type == SR_INST_SCPI)
+                               sr_scpi_free(sdi->conn);
                }
                if (clear_private)
                        clear_private(sdi->priv);
-               sdi = l->data;
+               else
+                       g_free(sdi->priv);
                sr_dev_inst_free(sdi);
        }
 
@@ -216,3 +285,4 @@ SR_PRIV int std_dev_clear(const struct sr_dev_driver *driver,
 
        return ret;
 }
+
@@ -101,27 +106,81 @@ SR_PRIV int std_session_send_df_header(const struct sr_dev_inst *sdi,
  *            Must not be NULL.
  * @param cb_data Opaque 'cb_data' pointer. Must not be NULL.
- * @param hw_dev_close_fn Function pointer to the driver's hw_dev_close().
+ * @param dev_close_fn Function pointer to the driver's dev_close().
  *                       Must not be NULL.
  * @param serial The serial device instance (struct serial_dev_inst *).
@@ -101,27 +106,81 @@ SR_PRIV int std_session_send_df_header(const struct sr_dev_inst *sdi,
  *               Must not be NULL. An empty string is allowed.
  *
- * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
- *         SR_ERR upon other errors.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_ARG Invalid arguments.
+ * @retval SR_ERR_DEV_CLOSED Device is closed.
+ * @retval SR_ERR Other errors.
  */